@@ -59,14 +59,11 @@ class Function(Field):
                 return
         setattr(self._field, name, value)
 
-    def search(self, model, name, clause):
-        '''
-        Call the searcher.
-        Return a list of clauses.
-        '''
+    def convert_domain(self, domain, tables, Model):
+        name, operator, value = domain[:3]
         if not self.searcher:
-            model.raise_user_error('search_function_missing', name)
-        return getattr(model, self.searcher)(name, tuple(clause))
+            Model.raise_user_error('search_function_missing', name)
+        return getattr(Model, self.searcher)(name, domain)
 
     def get(self, ids, Model, name, values=None):
         '''
@@ -94,12 +91,16 @@ class Function(Field):
                 name = [name]
             return call(name)
 
-    def set(self, ids, Model, name, value):
+    def set(self, Model, name, ids, value, *args):
         '''
         Call the setter.
         '''
         if self.setter:
-            getattr(Model, self.setter)(Model.browse(ids), name, value)
+            # TODO change setter API to use sequence of records, value
+            setter = getattr(Model, self.setter)
+            args = iter((ids, value) + args)
+            for ids, value in zip(args, args):
+                setter(Model.browse(ids), name, value)
 
     def __set__(self, inst, value):
         self._field.__set__(inst, value)
