@@ -138,18 +138,19 @@ class ModelStorage(Model):
         ModelAccess = pool.get('ir.model.access')
         ModelFieldAccess = pool.get('ir.model.field.access')
 
-        ModelAccess.check(cls.__name__, 'write')
-        ModelFieldAccess.check(cls.__name__,
-            [x for x in values if x in cls._fields], 'write')
-
         assert not len(args) % 2
         actions = iter((records, values) + args)
         all_records = []
+        all_fields = set()
         for records, values in zip(actions, actions):
             if not cls.check_xml_record(records, values):
                 cls.raise_user_error('write_xml_record',
                         error_description='xml_record_desc')
             all_records += records
+            all_fields.update(values.iterkeys())
+
+        ModelAccess.check(cls.__name__, 'write')
+        ModelFieldAccess.check(cls.__name__, all_fields, 'write')
 
         # Increase transaction counter
         Transaction().counter += 1
@@ -288,9 +289,18 @@ class ModelStorage(Model):
                 del data['id']
             return data
 
+        # Reset MPTT field to the default value
+        mptt = set()
+        for field in cls._fields.itervalues():
+            if (isinstance(field, fields.Many2One)
+                    and field.model_name == cls.__name__
+                    and field.left and field.right):
+                mptt.add(field.left)
+                mptt.add(field.right)
         fields_names = [n for n, f in cls._fields.iteritems()
             if (not isinstance(f, fields.Function)
-                or isinstance(f, fields.Property))]
+                or isinstance(f, fields.Property))
+            and n not in mptt]
         ids = map(int, records)
         datas = cls.read(ids, fields_names=fields_names)
         datas = dict((d['id'], d) for d in datas)
